import { defineNuxtModule, createResolver, addPluginTemplate, addTemplate, addImports } from '@nuxt/kit';
import { withHttps } from 'ufo';
import { defu } from 'defu';

const name = "@innodata/nuxtjs-alt-http";
const version = "1.5.3";

const CONFIG_KEY = "http";
const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: CONFIG_KEY,
    compatibility: {
      nuxt: "^3.0.0"
    }
  },
  defaults: {},
  setup(opts, nuxt) {
    const moduleOptions = defu(nuxt.options.runtimeConfig?.public?.http, opts);
    const defaultHost = moduleOptions.host || process.env.NITRO_HOST || process.env.HOST || "localhost";
    const defaultPort = moduleOptions.port || process.env.NITRO_PORT || process.env.PORT || 3e3;
    const prefix = moduleOptions.prefix || process.env.PREFIX || "/";
    const options = defu(moduleOptions, {
      baseURL: `http://${defaultHost}:${defaultPort}${prefix}`,
      browserBaseURL: void 0,
      proxyHeaders: true,
      proxyHeadersIgnore: [
        "accept",
        "connection",
        "cf-connecting-ip",
        "cf-ray",
        "content-length",
        "content-md5",
        "content-type",
        "host",
        "if-modified-since",
        "if-none-match",
        "x-forwarded-host",
        "x-forwarded-port",
        "x-forwarded-proto"
      ],
      serverTimeout: 1e4,
      clientTimeout: 25e3,
      https: false,
      retry: 1,
      headers: {
        accept: "application/json, text/plain, */*"
      },
      credentials: "omit",
      debug: false,
      interceptorPlugin: false
    });
    if (typeof options.browserBaseURL === "undefined") {
      options.browserBaseURL = nuxt.options.app.baseURL;
    }
    if (options.https === true) {
      options.baseURL = withHttps(options.baseURL);
      options.browserBaseURL = withHttps(options.browserBaseURL);
    }
    const resolver = createResolver(import.meta.url);
    if (Object.hasOwn(nuxt.options, "proxy") && moduleOptions.interceptorPlugin) {
      addPluginTemplate({
        src: resolver.resolve("runtime/templates/interceptor.plugin.mjs"),
        filename: "proxy.plugin.mjs",
        // @ts-ignore
        options: nuxt.options.proxy
      });
    }
    addPluginTemplate({
      src: resolver.resolve("runtime/templates/http.plugin.mjs"),
      filename: "http.plugin.mjs",
      options
    });
    addTemplate({
      getContents: () => getHttpDTS(),
      filename: "http.plugin.d.ts",
      write: true
    });
    const composables = resolver.resolve("runtime/composables");
    addImports([
      { from: composables, name: "useHttp" },
      { from: composables, name: "useLazyHttp" }
    ]);
    if (process.platform !== "win32") {
      addTemplate({
        getContents: () => nitroHttp(),
        filename: `nitro-http.mjs`,
        write: true
      });
      nuxt.hook("nitro:config", (nitro) => {
        nitro.plugins = nitro.plugins || [];
        nitro.plugins.push(resolver.resolve(nuxt.options.buildDir, `nitro-http.mjs`));
      });
    }
  }
});
function nitroHttp() {
  return `import { createInstance } from '@refactorjs/ofetch'

export default function (nitroApp) {
    // should inherit defaults from $fetch
    globalThis.$http = createInstance({}, $fetch)
}
`;
}
function getHttpDTS() {
  return `import type { Plugin } from '#app'
import { FetchInstance } from '@refactorjs/ofetch'

declare const _default: Plugin<{
    http: FetchInstance;
}>;

export default _default;
`;
}

export { module as default };
